/* lib.c */

#include <stdlib.h>
#include <sys/types.h>
#include <string.h>
#include <unistd.h>

#include <gtk/gtk.h>

#include "main.h"
#include "dialog.h"
#include "lib.h"

unsigned char kanji2nd;

unsigned char isAcceptable[96] =

/*	Bit 0		xalpha		-- see HTFile.h
**	Bit 1		xpalpha 	-- as xalpha but with plus.
**	Bit 3 ...	path		-- as xpalphas but with /
*/
    /*	 0 1 2 3 4 5 6 7 8 9 A B C D E F */
    {	 0,0,0,0,0,0,0,0,0,0,7,6,0,7,7,4,	/* 2x	!"#$%&'()*+,-./  */
	 7,7,7,7,7,7,7,7,7,7,0,0,0,0,0,0,	/* 3x  0123456789:;<=>?  */
	 7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,	/* 4x  @ABCDEFGHIJKLMNO  */
	 7,7,7,7,7,7,7,7,7,7,7,0,0,0,0,7,	/* 5X  PQRSTUVWXYZ[\]^_  */
	 0,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,	/* 6x  `abcdefghijklmno  */
	 7,7,7,7,7,7,7,7,7,7,7,0,0,0,0,0 };	/* 7X  pqrstuvwxyz{|}~	DEL */

char *hex = "0123456789ABCDEF";

/* convert JIS code to Shift-JIS code */
unsigned char jistojms(unsigned char c1, unsigned char c2)
{
    if (c1 & 1)
      {
        c1 = (c1 >> 1) + 0x71;
        c2 += 0x1f;
        if (c2 >= 0x7f)
	  c2++;
      }
    else
      {
        c1 = (c1 >> 1) + 0x70;
        c2 += 0x7e;
      }
    if (c1 > 0x9f)
      c1 += 0x40;
    kanji2nd = c2;
    return c1;
}

/* EUC to Shift-JIS */
void euctosjis(unsigned char *src, unsigned char *dst)
{
    unsigned char *inbufptr, *outbufptr, c, c2;

    inbufptr = src;
    outbufptr = dst;

    if ((c = *inbufptr++) == '\0')
        return;
    while (1)
      {
        if (c < 0x80)
	  {
	    if (c == '\n')
	      {
	        *outbufptr++ = '\r';
                *outbufptr++ = '\n';
	      }
	    else
	      {
	        *outbufptr++ = c;
	      }
	  }
        else if (iseuc(c))
	  {
            if ((c2 = *inbufptr++) == '\0')
	      {
                *outbufptr++ = c;
                break;
	      }
            if (iseuc(c2))
	      {
                c = jistojms(c & 0x7f, c2 & 0x7f);
                *outbufptr++ = c;
                *outbufptr++ = kanji2nd;
	      }
            else
	      {
                *outbufptr++ = c;
                *outbufptr++ = c2;
	      }
	  }
        else if (c == 0x8e)
	  {
            if ((c = *inbufptr++) == '\0')
	      break;
            *outbufptr++ = c;
	  }
        else if (c == 0x8f)
	  {
            if ((c = *inbufptr++) == '\0')
	      break;
            *outbufptr++ = c;
            if ((c = *inbufptr++) == '\0')
	      break;
            *outbufptr++ = c;
	  }
        else
	  *outbufptr++ = c;
        if ((c = *inbufptr++) == '\0')
	  break;
      }
    *outbufptr++ = '\0';
    return;
}

void to_escape (unsigned char *src, unsigned char *dst)
{
    unsigned char *p;
    unsigned char *q;
    unsigned char *src2;
    int unacceptable = 0;

    euctosjis(src, dst);
    src2 = safe_strdup(dst);

    for (p = src2; *p; p++)
      if (!(*p == ' ' || ACCEPTABLE((unsigned char)TOASCII(*p))))
	    unacceptable++;
    for (q = dst, p = src2; *p; p++)
      {
	unsigned char a = TOASCII(*p);
	if (a == 32)
	  {
	    *q++ = '+';
	  }
	else if (!ACCEPTABLE(a))
	  {
	    *q++ = HEX_ESCAPE;	/* Means hex commming */
	    *q++ = hex[a >> 4];
	    *q++ = hex[a & 15];
	  }
	else
	  {
	    *q++ = *p;
	  }
      }
    *q++ = '\0';		/* Terminate */
    safe_free((void **) &src2);
}

char *safe_strdup (const char *s)
{
    char *p;
    size_t l;

    if (!s || !*s) return 0;
    l = strlen (s) + 1;
    p = (char *)safe_malloc (l);
    memcpy (p, s, l);
    return (p);
}

void *safe_malloc (unsigned int siz)
{
    void *p;

    if (siz == 0)
      return 0;
    if ((p = (void *) malloc (siz)) == 0)
      {
	sleep (1);
	do_exit (1);
      }
    return (p);
}

void safe_realloc (void **p, size_t siz)
{
    void *r;

    if (siz == 0)
      {
	if (*p)
	  {
	    free (*p);
	    *p = NULL;
	  }
	return;
      }

    if (*p)
      r = (void *) realloc (*p, siz);
    else
      {
	/* realloc(NULL, nbytes) doesn't seem to work under SunOS 4.1.x */
	r = (void *) malloc (siz);
      }

    if (!r)
      {
	alert_dialog("$B%a%b%j$,B-$j$^$;$s(B", "OK", NULL, NULL);
	sleep (1);
	do_exit (1);
      }

    *p = r;
}

void safe_free (void **p)
{
    if (*p)
      {
	free (*p);
	*p = 0;
      }
}
