/* menu.c */

#include <gtk/gtk.h>
#include <string.h>

#include "main.h"
#include "menu.h"
#include "dialog.h"
#include "lib.h"
#include "config.h"

#ifndef GTK_HAVE_FEATURES_1_1_0
static void menus_remove_accel(GtkWidget * widget, gchar * signal_name, gchar * path);
static gint menus_install_accel(GtkWidget * widget, gchar * signal_name, gchar key, gchar modifiers, gchar * path);
void menus_init(void);
void menus_create(GtkMenuEntry * entries, int nmenu_entries);

static GtkMenuEntry menu_items[] =
{
  {"<Main>/ե/λ", "<control>X", menu_quit, NULL},
  {"<Main>/Խ/","<control>E", menu_config, NULL },
  {"<Main>/إ/DCUPȤϡ...", "<control>H", menu_about, NULL}
};

static int nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
static int initialize = TRUE;
static GtkMenuFactory *factory = NULL;
static GtkMenuFactory *subfactory[1];
static GHashTable *entry_ht = NULL;

void init_menu()
{
  GtkWidget *menubar;
  GtkAcceleratorTable *accel;

  if (initialize)
    menus_init();

  menubar = subfactory[0]->widget;
  accel = subfactory[0]->table;

  gtk_window_add_accelerator_table(GTK_WINDOW(window), accel);
  gtk_box_pack_start(GTK_BOX(main_vbox), menubar, FALSE, TRUE, 0);
  gtk_widget_show(menubar);
}

void menus_init(void)
{
  if (initialize)
    {
      initialize = FALSE;

      factory = gtk_menu_factory_new(GTK_MENU_FACTORY_MENU_BAR);
      subfactory[0] = gtk_menu_factory_new(GTK_MENU_FACTORY_MENU_BAR);

      gtk_menu_factory_add_subfactory(factory, subfactory[0], "<Main>");
      menus_create(menu_items, nmenu_items);
    }
}

void menus_create(GtkMenuEntry * entries, int nmenu_entries)
{
  char *accelerator;
  int i;

  if (initialize)
    menus_init();

  if (entry_ht)
    for (i = 0; i < nmenu_entries; i++)
      {
	accelerator = g_hash_table_lookup(entry_ht, entries[i].path);
	if (accelerator)
	  {
	    if (accelerator[0] == '\0')
	      entries[i].accelerator = NULL;
	    else
	      entries[i].accelerator = accelerator;
	  }
      }
  gtk_menu_factory_add_entries(factory, entries, nmenu_entries);

  for (i = 0; i < nmenu_entries; i++)
    if (entries[i].widget)
      {
	gtk_signal_connect(GTK_OBJECT(entries[i].widget),
			   "install_accelerator",
			   (GtkSignalFunc) menus_install_accel,
			   entries[i].path);
	gtk_signal_connect(GTK_OBJECT(entries[i].widget),
			   "remove_accelerator",
			   (GtkSignalFunc) menus_remove_accel,
			   entries[i].path);
      }
}

static gint menus_install_accel(GtkWidget * widget, gchar * signal_name,
				gchar key, gchar modifiers, gchar * path)
{
  char accel[64];
  char *t1, t2[2];
    
  accel[0] = '\0';
  if (modifiers & GDK_CONTROL_MASK)
    strcat(accel, "<control>");
  if (modifiers & GDK_SHIFT_MASK)
    strcat(accel, "<shift>");
  if (modifiers & GDK_MOD1_MASK)
    strcat(accel, "<alt>");

  t2[0] = key;
  t2[1] = '\0';
  strcat(accel, t2);

  if (entry_ht)
    {
      t1 = g_hash_table_lookup(entry_ht, path);
      g_free(t1);
    }
  else
    entry_ht = g_hash_table_new(g_str_hash, g_str_equal);

  g_hash_table_insert(entry_ht, path, g_strdup(accel));
  
  return TRUE;
}

static void menus_remove_accel(GtkWidget * widget,
			       gchar * signal_name,
			       gchar * path)
{
  char *t;

  if (entry_ht)
    {
      t = g_hash_table_lookup(entry_ht, path);
      g_free(t);
      g_hash_table_insert(entry_ht, path, g_strdup(""));
    }
}
#else
static GtkItemFactoryEntry menu_items[] =
{
  {"/ե", NULL, NULL, 0, "<Branch>"},
  {"/ե/λ", "<control>X", menu_quit, 0, NULL},
  {"/Խ", NULL, NULL, 0, "<Branch>" },
  {"/Խ/","<control>E", menu_config, 0, NULL },
  {"/إ", NULL, NULL, 0, "<Branch>"},
  {"/إ/DCUPȤϡ...", "<control>H", menu_about, 0, NULL}
};

static GtkWidget *menubar = NULL;

void init_menu()
{
  int nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;

  accel_group = gtk_accel_group_new();

  item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<Main>", 
				       accel_group);

  gtk_item_factory_create_items(item_factory, nmenu_items, menu_items, NULL);

  gtk_accel_group_attach (accel_group, GTK_OBJECT (window));

  menubar = gtk_item_factory_get_widget(item_factory, "<Main>");

  gtk_box_pack_start(GTK_BOX(main_vbox), menubar, FALSE, TRUE, 0);
  gtk_widget_show(menubar);
}
#endif

void menu_quit(GtkWidget *widget, gpointer data)
{
  do_exit(0);
}

void menu_about(GtkWidget *widget, gpointer data)
{
  alert_dialog("CCUP礭ECUP\n꾮餷Ǥ()", "OK", NULL, NULL);
}

void menu_config(GtkWidget *widget, gpointer data)
{
  config();
}
